@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2017 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -12,7 +12,7 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ *   along with this program; if not, see <https://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
 #include "onlinesearchmathscinet.h"
@@ -21,13 +21,14 @@
 #include <QNetworkRequest>
 #include <QNetworkAccessManager>
 #include <QMap>
+#include <QUrlQuery>
 
-#include <KLocale>
-#include <KDebug>
+#include <KLocalizedString>
 
 #include "fileimporterbibtex.h"
-#include "kbibtexnamespace.h"
+#include "kbibtex.h"
 #include "internalnetworkaccessmanager.h"
+#include "logging_networking.h"
 
 class OnlineSearchMathSciNet::OnlineSearchMathSciNetPrivate
 {
@@ -48,8 +49,8 @@ public:
     }
 };
 
-const QString OnlineSearchMathSciNet::OnlineSearchMathSciNetPrivate::queryFormUrl = QLatin1String("http://www.ams.org/mathscinet/");
-const QString OnlineSearchMathSciNet::OnlineSearchMathSciNetPrivate::queryUrlStem = QLatin1String("http://www.ams.org/mathscinet/search/publications.html?client=KBibTeX");
+const QString OnlineSearchMathSciNet::OnlineSearchMathSciNetPrivate::queryFormUrl = QStringLiteral("http://www.ams.org/mathscinet/");
+const QString OnlineSearchMathSciNet::OnlineSearchMathSciNetPrivate::queryUrlStem = QStringLiteral("http://www.ams.org/mathscinet/search/publications.html?client=KBibTeX");
 
 OnlineSearchMathSciNet::OnlineSearchMathSciNet(QWidget *parent)
         : OnlineSearchAbstract(parent), d(new OnlineSearchMathSciNetPrivate(this))
@@ -65,58 +66,50 @@ OnlineSearchMathSciNet::~OnlineSearchMathSciNet()
 void OnlineSearchMathSciNet::startSearch(const QMap<QString, QString> &query, int numResults)
 {
     m_hasBeenCanceled = false;
+    emit progress(curStep = 0, numSteps = 3);
 
     d->queryParameters.clear();
     d->numResults = qMin(50, numResults); /// limit query to max 50 elements
     int index = 1;
 
     const QString freeText = query[queryKeyFreeText];
-    QStringList elements = splitRespectingQuotationMarks(freeText);
-    foreach (const QString &element, elements) {
-        d->queryParameters.insert(QString(QLatin1String("pg%1")).arg(index), QLatin1String("ALLF"));
-        d->queryParameters.insert(QString(QLatin1String("s%1")).arg(index), element);
+    const QStringList elementsFreeText = splitRespectingQuotationMarks(freeText);
+    for (const QString &element : elementsFreeText) {
+        d->queryParameters.insert(QString(QStringLiteral("pg%1")).arg(index), QStringLiteral("ALLF"));
+        d->queryParameters.insert(QString(QStringLiteral("s%1")).arg(index), element);
         ++index;
     }
 
     const QString title = query[queryKeyTitle];
-    elements = splitRespectingQuotationMarks(title);
-    foreach (const QString &element, elements) {
-        d->queryParameters.insert(QString(QLatin1String("pg%1")).arg(index), QLatin1String("TI"));
-        d->queryParameters.insert(QString(QLatin1String("s%1")).arg(index), element);
+    const QStringList elementsTitle = splitRespectingQuotationMarks(title);
+    for (const QString &element : elementsTitle) {
+        d->queryParameters.insert(QString(QStringLiteral("pg%1")).arg(index), QStringLiteral("TI"));
+        d->queryParameters.insert(QString(QStringLiteral("s%1")).arg(index), element);
         ++index;
     }
 
     const QString authors = query[queryKeyAuthor];
-    elements = splitRespectingQuotationMarks(authors);
-    foreach (const QString &element, elements) {
-        d->queryParameters.insert(QString(QLatin1String("pg%1")).arg(index), QLatin1String("ICN"));
-        d->queryParameters.insert(QString(QLatin1String("s%1")).arg(index), element);
+    const QStringList elementsAuthor = splitRespectingQuotationMarks(authors);
+    for (const QString &element : elementsAuthor) {
+        d->queryParameters.insert(QString(QStringLiteral("pg%1")).arg(index), QStringLiteral("ICN"));
+        d->queryParameters.insert(QString(QStringLiteral("s%1")).arg(index), element);
         ++index;
     }
 
     const QString year = query[queryKeyYear];
     if (year.isEmpty()) {
-        d->queryParameters.insert(QLatin1String("dr"), QLatin1String("all"));
+        d->queryParameters.insert(QStringLiteral("dr"), QStringLiteral("all"));
     } else {
-        d->queryParameters.insert(QLatin1String("dr"), QLatin1String("pubyear"));
-        d->queryParameters.insert(QLatin1String("yrop"), QLatin1String("eq"));
-        d->queryParameters.insert(QLatin1String("arg3"), year);
+        d->queryParameters.insert(QStringLiteral("dr"), QStringLiteral("pubyear"));
+        d->queryParameters.insert(QStringLiteral("yrop"), QStringLiteral("eq"));
+        d->queryParameters.insert(QStringLiteral("arg3"), year);
     }
 
-    emit progress(0, 3);
-
     /// issue request for start page
     QNetworkRequest request(OnlineSearchMathSciNetPrivate::queryFormUrl);
-    QNetworkReply *reply = InternalNetworkAccessManager::self()->get(request);
-    InternalNetworkAccessManager::self()->setNetworkReplyTimeout(reply);
-    connect(reply, SIGNAL(finished()), this, SLOT(doneFetchingQueryForm()));
-}
-
-void OnlineSearchMathSciNet::startSearch()
-{
-    d->queryParameters.clear();
-    m_hasBeenCanceled = false;
-    delayedStoppedSearch(resultNoError);
+    QNetworkReply *reply = InternalNetworkAccessManager::instance().get(request);
+    InternalNetworkAccessManager::instance().setNetworkReplyTimeout(reply);
+    connect(reply, &QNetworkReply::finished, this, &OnlineSearchMathSciNet::doneFetchingQueryForm);
 }
 
 QString OnlineSearchMathSciNet::label() const
@@ -126,29 +119,19 @@ QString OnlineSearchMathSciNet::label() const
 
 QString OnlineSearchMathSciNet::favIconUrl() const
 {
-    return QLatin1String("http://www.ams.org/favicon.ico");
+    return QStringLiteral("http://www.ams.org/favicon.ico");
 }
 
-OnlineSearchQueryFormAbstract *OnlineSearchMathSciNet::customWidget(QWidget *)
+QUrl OnlineSearchMathSciNet::homepage() const
 {
-    return NULL;
-}
-
-KUrl OnlineSearchMathSciNet::homepage() const
-{
-    return KUrl("http://www.ams.org/mathscinet/help/about.html");
-}
-
-void OnlineSearchMathSciNet::cancel()
-{
-    OnlineSearchAbstract::cancel();
+    return QUrl(QStringLiteral("http://www.ams.org/mathscinet/help/about.html"));
 }
 
 void OnlineSearchMathSciNet::doneFetchingQueryForm()
 {
     QNetworkReply *reply = static_cast<QNetworkReply *>(sender());
 
-    emit progress(1, 3);
+    emit progress(++curStep, numSteps);
 
     if (handleErrors(reply)) {
         // UNUSED const QString htmlText = QString::fromUtf8(reply->readAll().constData());
@@ -160,67 +143,72 @@ void OnlineSearchMathSciNet::doneFetchingQueryForm()
             formParams.insert(it.key(), it.value());
 
         /// build url by appending parameters
-        KUrl url(OnlineSearchMathSciNetPrivate::queryUrlStem);
+        QUrl url(OnlineSearchMathSciNetPrivate::queryUrlStem);
+        QUrlQuery query(url);
         for (QMap<QString, QString>::ConstIterator it = formParams.constBegin(); it != formParams.constEnd(); ++it)
-            url.addQueryItem(it.key(), it.value());
+            query.addQueryItem(it.key(), it.value());
         for (int i = 1; i <= d->queryParameters.count(); ++i)
-            url.addQueryItem(QString(QLatin1String("co%1")).arg(i), QLatin1String("AND")); ///< join search terms with an AND operation
+            query.addQueryItem(QString(QStringLiteral("co%1")).arg(i), QStringLiteral("AND")); ///< join search terms with an AND operation
+        url.setQuery(query);
 
         /// issue request for result page
         QNetworkRequest request(url);
-        QNetworkReply *newReply = InternalNetworkAccessManager::self()->get(request, reply);
-        InternalNetworkAccessManager::self()->setNetworkReplyTimeout(newReply);
-        connect(newReply, SIGNAL(finished()), this, SLOT(doneFetchingResultPage()));
+        QNetworkReply *newReply = InternalNetworkAccessManager::instance().get(request, reply);
+        InternalNetworkAccessManager::instance().setNetworkReplyTimeout(newReply);
+        connect(newReply, &QNetworkReply::finished, this, &OnlineSearchMathSciNet::doneFetchingResultPage);
     } else
-        kDebug() << "url was" << reply->url().toString();
+        qCWarning(LOG_KBIBTEX_NETWORKING) << "url was" << reply->url().toDisplayString();
 }
 
 void OnlineSearchMathSciNet::doneFetchingResultPage()
 {
     QNetworkReply *reply = static_cast<QNetworkReply *>(sender());
 
-    emit progress(2, 3);
+    emit progress(++curStep, numSteps);
 
     if (handleErrors(reply)) {
         const QString htmlText = QString::fromUtf8(reply->readAll().constData());
 
         /// extract form's parameters ...
-        QMap<QString, QString> formParams = formParameters(htmlText, htmlText.indexOf(QLatin1String("<form name=\"batchDownload\" action="), Qt::CaseInsensitive));
+        QMap<QString, QString> formParams = formParameters(htmlText, htmlText.indexOf(QStringLiteral("<form name=\"batchDownload\" action="), Qt::CaseInsensitive));
 
         /// build url by appending parameters
-        KUrl url(OnlineSearchMathSciNetPrivate::queryUrlStem);
-        QStringList copyParameters = QStringList() << QLatin1String("foo") << QLatin1String("reqargs") << QLatin1String("batch_title");
-        foreach (const QString &param, copyParameters)
-            url.addQueryItem(param, formParams[param]);
-        url.addQueryItem(QLatin1String("fmt"), QLatin1String("bibtex"));
+        QUrl url(OnlineSearchMathSciNetPrivate::queryUrlStem);
+        QUrlQuery query(url);
+        const QStringList copyParameters = QStringList() << QStringLiteral("foo") << QStringLiteral("reqargs") << QStringLiteral("batch_title");
+        for (const QString &param : copyParameters) {
+            query.addQueryItem(param, formParams[param]);
+        }
+        query.addQueryItem(QStringLiteral("fmt"), QStringLiteral("bibtex"));
 
         int p = -1, count = 0;
-        static const QRegExp checkBoxRegExp(QLatin1String("<input class=\"hlCheckBox\" type=\"checkbox\" name=\"b\" value=\"(\\d+)\""));
+        static const QRegExp checkBoxRegExp(QStringLiteral("<input class=\"hlCheckBox\" type=\"checkbox\" name=\"b\" value=\"(\\d+)\""));
         while (count < d->numResults && (p = checkBoxRegExp.indexIn(htmlText, p + 1)) >= 0) {
-            url.addQueryItem(QLatin1String("b"), checkBoxRegExp.cap(1));
+            query.addQueryItem(QStringLiteral("b"), checkBoxRegExp.cap(1));
             ++count;
         }
 
+        url.setQuery(query);
         if (count > 0) {
             /// issue request for bibtex code
             QNetworkRequest request(url);
-            QNetworkReply *newReply = InternalNetworkAccessManager::self()->get(request, reply);
-            InternalNetworkAccessManager::self()->setNetworkReplyTimeout(newReply);
-            connect(newReply, SIGNAL(finished()), this, SLOT(doneFetchingBibTeXcode()));
+            QNetworkReply *newReply = InternalNetworkAccessManager::instance().get(request, reply);
+            InternalNetworkAccessManager::instance().setNetworkReplyTimeout(newReply);
+            connect(newReply, &QNetworkReply::finished, this, &OnlineSearchMathSciNet::doneFetchingBibTeXcode);
         } else {
             /// nothing found
-            emit progress(3, 3);
-            emit stoppedSearch(resultNoError);
+            emit progress(curStep = numSteps, numSteps);
+            stopSearch(resultNoError);
         }
     } else
-        kDebug() << "url was" << reply->url().toString();
+        qCWarning(LOG_KBIBTEX_NETWORKING) << "url was" << reply->url().toDisplayString();
 }
 
 void OnlineSearchMathSciNet::doneFetchingBibTeXcode()
 {
     QNetworkReply *reply = static_cast<QNetworkReply *>(sender());
 
-    emit progress(3, 3);
+    emit progress(curStep = numSteps, numSteps);
 
     if (handleErrors(reply)) {
         /// ensure proper treatment of UTF-8 characters
@@ -228,32 +216,32 @@ void OnlineSearchMathSciNet::doneFetchingBibTeXcode()
 
         QString bibtexCode;
         int p1 = -1, p2 = -1;
-        while ((p1 = htmlCode.indexOf(QLatin1String("<pre>"), p2 + 1)) >= 0 && (p2 = htmlCode.indexOf(QLatin1String("</pre>"), p1 + 1)) >= 0) {
+        while ((p1 = htmlCode.indexOf(QStringLiteral("<pre>"), p2 + 1)) >= 0 && (p2 = htmlCode.indexOf(QStringLiteral("</pre>"), p1 + 1)) >= 0) {
             bibtexCode += htmlCode.midRef(p1 + 5, p2 - p1 - 5);
             bibtexCode += QLatin1Char('\n');
         }
 
-        FileImporterBibTeX importer;
-        File *bibtexFile = importer.fromString(bibtexCode);
+        FileImporterBibTeX importer(this);
+        const File *bibtexFile = importer.fromString(bibtexCode);
 
         bool hasEntry = false;
-        if (bibtexFile != NULL) {
-            for (File::ConstIterator it = bibtexFile->constBegin(); it != bibtexFile->constEnd(); ++it) {
-                QSharedPointer<Entry> entry = (*it).dynamicCast<Entry>();
+        if (bibtexFile != nullptr) {
+            for (const auto &element : *bibtexFile) {
+                QSharedPointer<Entry> entry = element.dynamicCast<Entry>();
                 hasEntry |= publishEntry(entry);
 
             }
             delete bibtexFile;
         }
 
-        emit stoppedSearch(hasEntry ? resultNoError : resultUnspecifiedError);
+        stopSearch(hasEntry ? resultNoError : resultUnspecifiedError);
     } else
-        kDebug() << "url was" << reply->url().toString();
+        qCWarning(LOG_KBIBTEX_NETWORKING) << "url was" << reply->url().toDisplayString();
 }
 
 void OnlineSearchMathSciNet::sanitizeEntry(QSharedPointer<Entry> entry)
 {
-    const QString ftFJournal = QLatin1String("fjournal");
+    const QString ftFJournal = QStringLiteral("fjournal");
     if (entry->contains(ftFJournal)) {
         Value v = entry->value(ftFJournal);
         entry->remove(Entry::ftJournal);
