@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2017 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -12,7 +12,7 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ *   along with this program; if not, see <https://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
 #include "searchform.h"
@@ -23,25 +23,27 @@
 #include <QListWidget>
 #include <QSpinBox>
 #include <QStackedWidget>
+#include <QTabWidget>
 #include <QProgressBar>
+#include <QMimeDatabase>
+#include <QMimeType>
 #include <QTimer>
 #include <QSet>
+#include <QAction>
 #include <QScrollArea>
+#include <QIcon>
+#include <QPushButton>
+#include <QDebug>
 
-#include <KPushButton>
 #include <KLineEdit>
-#include <KLocale>
-#include <KIcon>
-#include <KDebug>
+#include <KLocalizedString>
 #include <KRun>
-#include <KMimeType>
 #include <KMessageBox>
-#include <KTemporaryFile>
-#include <KAction>
-#include <kparts/part.h>
-#include <KStandardDirs>
+#include <KParts/Part>
+#include <KParts/ReadOnlyPart>
 #include <KConfigGroup>
-#include <KTabWidget>
+#include <KSharedConfig>
+#include <kio_version.h>
 
 #include "element.h"
 #include "file.h"
@@ -70,12 +72,9 @@
 #include "onlinesearchbiorxiv.h"
 #include "openfileinfo.h"
 #include "fileview.h"
-#include "filemodel.h"
+#include "models/filemodel.h"
 #include "searchresults.h"
-
-const int HomepageRole = Qt::UserRole + 5;
-const int WidgetRole = Qt::UserRole + 6;
-const int NameRole = Qt::UserRole + 7;
+#include "logging_program.h"
 
 class SearchForm::SearchFormPrivate
 {
@@ -85,7 +84,7 @@ private:
     QWidget *listContainer;
     QListWidget *enginesList;
     QLabel *whichEnginesLabel;
-    KAction *actionOpenHomepage;
+    QAction *actionOpenHomepage;
 
 public:
     KSharedConfigPtr config;
@@ -94,24 +93,33 @@ public:
     SearchResults *sr;
     QMap<QListWidgetItem *, OnlineSearchAbstract *> itemToOnlineSearch;
     QSet<OnlineSearchAbstract *> runningSearches;
-    KPushButton *searchButton;
-    KPushButton *useEntryButton;
+    QPushButton *searchButton;
+    QPushButton *useEntryButton;
     OnlineSearchQueryFormGeneral *generalQueryTermsForm;
-    KTabWidget *tabWidget;
+    QTabWidget *tabWidget;
     QSharedPointer<const Entry> currentEntry;
     QProgressBar *progressBar;
     QMap<OnlineSearchAbstract *, int> progressMap;
     QMap<OnlineSearchQueryFormAbstract *, QScrollArea *> formToScrollArea;
 
+    enum SearchFormPrivateRole {
+        /// Homepage of a search engine
+        HomepageRole = Qt::UserRole + 5,
+        /// Special widget for a search engine
+        WidgetRole = Qt::UserRole + 6,
+        /// Name of a search engine
+        NameRole = Qt::UserRole + 7
+    };
+
     SearchFormPrivate(SearchResults *searchResults, SearchForm *parent)
-            : p(parent), whichEnginesLabel(NULL), config(KSharedConfig::openConfig(QLatin1String("kbibtexrc"))),
-          configGroupName(QLatin1String("Search Engines Docklet")), sr(searchResults), searchButton(NULL), useEntryButton(NULL), currentEntry(NULL) {
+            : p(parent), whichEnginesLabel(nullptr), config(KSharedConfig::openConfig(QStringLiteral("kbibtexrc"))),
+          configGroupName(QStringLiteral("Search Engines Docklet")), sr(searchResults), searchButton(nullptr), useEntryButton(nullptr), currentEntry(nullptr) {
         createGUI();
     }
 
     OnlineSearchQueryFormAbstract *currentQueryForm() {
         QScrollArea *area = qobject_cast<QScrollArea *>(queryTermsStack->currentWidget());
-        return formToScrollArea.key(area, NULL);
+        return formToScrollArea.key(area, nullptr);
     }
 
     QScrollArea *wrapInScrollArea(OnlineSearchQueryFormAbstract *form, QWidget *parent) {
@@ -134,7 +142,7 @@ public:
         whichEnginesLabel->setWordWrap(true);
         vLayout->addWidget(whichEnginesLabel);
         vLayout->setStretchFactor(whichEnginesLabel, 0);
-        connect(whichEnginesLabel, SIGNAL(linkActivated(QString)), p, SLOT(switchToEngines()));
+        connect(whichEnginesLabel, &QLabel::linkActivated, p, &SearchForm::switchToEngines);
 
         vLayout->addSpacing(8);
 
@@ -148,7 +156,7 @@ public:
         return container;
     }
 
-    OnlineSearchQueryFormAbstract *createGeneralQueryTermsForm(QWidget *parent = NULL) {
+    OnlineSearchQueryFormAbstract *createGeneralQueryTermsForm(QWidget *parent = nullptr) {
         generalQueryTermsForm = new OnlineSearchQueryFormGeneral(parent);
         return generalQueryTermsForm;
     }
@@ -161,12 +169,12 @@ public:
 
         enginesList = new QListWidget(listContainer);
         layout->addWidget(enginesList, 0, 0, 1, 1);
-        connect(enginesList, SIGNAL(itemChanged(QListWidgetItem*)), p, SLOT(itemCheckChanged(QListWidgetItem*)));
-        connect(enginesList, SIGNAL(currentItemChanged(QListWidgetItem*,QListWidgetItem*)), p, SLOT(enginesListCurrentChanged(QListWidgetItem*,QListWidgetItem*)));
+        connect(enginesList, &QListWidget::itemChanged, p, &SearchForm::itemCheckChanged);
+        connect(enginesList, &QListWidget::currentItemChanged, p, &SearchForm::enginesListCurrentChanged);
         enginesList->setSelectionMode(QAbstractItemView::NoSelection);
 
-        actionOpenHomepage = new KAction(KIcon("internet-web-browser"), i18n("Go to Homepage"), p);
-        connect(actionOpenHomepage, SIGNAL(triggered()), p, SLOT(openHomepage()));
+        actionOpenHomepage = new QAction(QIcon::fromTheme(QStringLiteral("internet-web-browser")), i18n("Go to Homepage"), p);
+        connect(actionOpenHomepage, &QAction::triggered, p, &SearchForm::openHomepage);
         enginesList->addAction(actionOpenHomepage);
         enginesList->setContextMenuPolicy(Qt::ActionsContextMenu);
 
@@ -182,31 +190,31 @@ public:
         layout->setColumnStretch(1, 1);
         layout->setColumnStretch(2, 0);
 
-        tabWidget = new KTabWidget(p);
+        tabWidget = new QTabWidget(p);
         tabWidget->setDocumentMode(true);
         layout->addWidget(tabWidget, 0, 0, 1, 3);
 
         QWidget *widget = createQueryTermsStack(tabWidget);
-        tabWidget->addTab(widget, KIcon("edit-rename"), i18n("Query Terms"));
+        tabWidget->addTab(widget, QIcon::fromTheme(QStringLiteral("edit-rename")), i18n("Query Terms"));
 
         QWidget *listContainer = createEnginesGUI(tabWidget);
-        tabWidget->addTab(listContainer, KIcon("applications-engineering"), i18n("Engines"));
+        tabWidget->addTab(listContainer, QIcon::fromTheme(QStringLiteral("applications-engineering")), i18n("Engines"));
 
-        connect(tabWidget, SIGNAL(currentChanged(int)), p, SLOT(tabSwitched(int)));
+        connect(tabWidget, &QTabWidget::currentChanged, p, &SearchForm::tabSwitched);
 
-        useEntryButton = new KPushButton(KIcon("go-up"), i18n("Use Entry"), p);
+        useEntryButton = new QPushButton(QIcon::fromTheme(QStringLiteral("go-up")), i18n("Use Entry"), p);
         layout->addWidget(useEntryButton, 1, 0, 1, 1);
         useEntryButton->setEnabled(false);
-        connect(useEntryButton, SIGNAL(clicked()), p, SLOT(copyFromEntry()));
+        connect(useEntryButton, &QPushButton::clicked, p, &SearchForm::copyFromEntry);
 
         progressBar = new QProgressBar(p);
         layout->addWidget(progressBar, 1, 1, 1, 1);
         progressBar->setMaximum(1000);
         progressBar->hide();
 
-        searchButton = new KPushButton(KIcon("edit-find"), i18n("Search"), p);
+        searchButton = new QPushButton(QIcon::fromTheme(QStringLiteral("edit-find")), i18n("Search"), p);
         layout->addWidget(searchButton, 1, 2, 1, 1);
-        connect(generalQueryTermsForm, SIGNAL(returnPressed()), searchButton, SLOT(click()));
+        connect(generalQueryTermsForm, &OnlineSearchQueryFormGeneral::returnPressed, searchButton, &QPushButton::click);
 
         updateGUI();
     }
@@ -234,7 +242,7 @@ public:
         addEngine(new OnlineSearchIDEASRePEc(p));
         addEngine(new OnlineSearchDOI(p));
 
-        p->itemCheckChanged(NULL);
+        p->itemCheckChanged(nullptr);
         updateGUI();
     }
 
@@ -250,37 +258,37 @@ public:
 
         OnlineSearchQueryFormAbstract *widget = engine->customWidget(queryTermsStack);
         item->setData(WidgetRole, QVariant::fromValue<OnlineSearchQueryFormAbstract *>(widget));
-        if (widget != NULL) {
-            connect(widget, SIGNAL(returnPressed()), searchButton, SLOT(click()));
+        if (widget != nullptr) {
+            connect(widget, &OnlineSearchQueryFormAbstract::returnPressed, searchButton, &QPushButton::click);
             QScrollArea *scrollArea = wrapInScrollArea(widget, queryTermsStack);
             queryTermsStack->addWidget(scrollArea);
         }
 
         itemToOnlineSearch.insert(item, engine);
-        connect(engine, SIGNAL(foundEntry(QSharedPointer<Entry>)), p, SLOT(foundEntry(QSharedPointer<Entry>)));
-        connect(engine, SIGNAL(stoppedSearch(int)), p, SLOT(stoppedSearch(int)));
-        connect(engine, SIGNAL(progress(int,int)), p, SLOT(updateProgress(int,int)));
+        connect(engine, &OnlineSearchAbstract::foundEntry, p, &SearchForm::foundEntry);
+        connect(engine, &OnlineSearchAbstract::stoppedSearch, p, &SearchForm::stoppedSearch);
+        connect(engine, &OnlineSearchAbstract::progress, p, &SearchForm::updateProgress);
     }
 
     void switchToSearch() {
         for (QMap<QListWidgetItem *, OnlineSearchAbstract *>::ConstIterator it = itemToOnlineSearch.constBegin(); it != itemToOnlineSearch.constEnd(); ++it)
-            disconnect(searchButton, SIGNAL(clicked()), it.value(), SLOT(cancel()));
+            disconnect(searchButton, &QPushButton::clicked, it.value(), &OnlineSearchAbstract::cancel);
 
-        connect(searchButton, SIGNAL(clicked()), p, SLOT(startSearch()));
+        connect(searchButton, &QPushButton::clicked, p, &SearchForm::startSearch);
         searchButton->setText(i18n("Search"));
-        searchButton->setIcon(KIcon("media-playback-start"));
+        searchButton->setIcon(QIcon::fromTheme(QStringLiteral("media-playback-start")));
         for (int i = tabWidget->count() - 1; i >= 0; --i)
             tabWidget->widget(i)->setEnabled(true);
         tabWidget->unsetCursor();
     }
 
     void switchToCancel() {
-        disconnect(searchButton, SIGNAL(clicked()), p, SLOT(startSearch()));
+        disconnect(searchButton, &QPushButton::clicked, p, &SearchForm::startSearch);
 
         for (QMap<QListWidgetItem *, OnlineSearchAbstract *>::ConstIterator it = itemToOnlineSearch.constBegin(); it != itemToOnlineSearch.constEnd(); ++it)
-            connect(searchButton, SIGNAL(clicked()), it.value(), SLOT(cancel()));
+            connect(searchButton, &QPushButton::clicked, it.value(), &OnlineSearchAbstract::cancel);
         searchButton->setText(i18n("Stop"));
-        searchButton->setIcon(KIcon("media-playback-stop"));
+        searchButton->setIcon(QIcon::fromTheme(QStringLiteral("media-playback-stop")));
         for (int i = tabWidget->count() - 1; i >= 0; --i)
             tabWidget->widget(i)->setEnabled(false);
         tabWidget->setCursor(Qt::WaitCursor);
@@ -291,10 +299,10 @@ public:
     }
 
     void updateGUI() {
-        if (whichEnginesLabel == NULL) return;
+        if (whichEnginesLabel == nullptr) return;
 
         QStringList checkedEngines;
-        QListWidgetItem *cursor = NULL;
+        QListWidgetItem *cursor = nullptr;
         for (QMap<QListWidgetItem *, OnlineSearchAbstract *>::ConstIterator it = itemToOnlineSearch.constBegin(); it != itemToOnlineSearch.constEnd(); ++it)
             if (it.key()->checkState() == Qt::Checked) {
                 checkedEngines << it.key()->text();
@@ -309,33 +317,37 @@ public:
         default: whichEnginesLabel->setText(i18n("Search engines <b>%1</b>, <b>%2</b>, and more are selected (<a href=\"changeEngine\">change</a>).", checkedEngines.first(), checkedEngines.at(1))); break;
         }
 
-        OnlineSearchQueryFormAbstract *currentQueryWidget = NULL;
-        if (cursor != NULL && checkedEngines.size() == 1)
+        OnlineSearchQueryFormAbstract *currentQueryWidget = nullptr;
+        if (cursor != nullptr && checkedEngines.size() == 1)
             currentQueryWidget = cursor->data(WidgetRole).value<OnlineSearchQueryFormAbstract *>();
-        if (currentQueryWidget == NULL)
+        if (currentQueryWidget == nullptr)
             currentQueryWidget = generalQueryTermsForm;
-        QScrollArea *area = formToScrollArea.value(currentQueryWidget, NULL);
-        if (area != NULL)
+        QScrollArea *area = formToScrollArea.value(currentQueryWidget, nullptr);
+        if (area != nullptr)
             queryTermsStack->setCurrentWidget(area);
 
-        if (useEntryButton != NULL)
+        if (useEntryButton != nullptr)
             useEntryButton->setEnabled(!currentEntry.isNull() && tabWidget->currentIndex() == 0);
     }
 
     void openHomepage() {
         QListWidgetItem *item = enginesList->currentItem();
-        if (item != NULL) {
-            KUrl url = item->data(HomepageRole).value<KUrl>();
+        if (item != nullptr) {
+            QUrl url = item->data(HomepageRole).toUrl();
             /// Guess mime type for url to open
-            KMimeType::Ptr mimeType = FileInfo::mimeTypeForUrl(url);
-            const QString mimeTypeName = mimeType->name();
+            QMimeType mimeType = FileInfo::mimeTypeForUrl(url);
+            const QString mimeTypeName = mimeType.name();
             /// Ask KDE subsystem to open url in viewer matching mime type
+#if KIO_VERSION < 0x051f00 // < 5.31.0
             KRun::runUrl(url, mimeTypeName, p, false, false);
+#else // KIO_VERSION < 0x051f00 // >= 5.31.0
+            KRun::runUrl(url, mimeTypeName, p, KRun::RunFlags());
+#endif // KIO_VERSION < 0x051f00
         }
     }
 
     void enginesListCurrentChanged(QListWidgetItem *current) {
-        actionOpenHomepage->setEnabled(current != NULL);
+        actionOpenHomepage->setEnabled(current != nullptr);
     }
 };
 
@@ -356,7 +368,7 @@ void SearchForm::updatedConfiguration()
     d->loadEngines();
 }
 
-void SearchForm::setElement(QSharedPointer<Element> element, File *)
+void SearchForm::setElement(QSharedPointer<Element> element, const File *)
 {
     d->currentEntry = element.dynamicCast<const Entry>();
     d->useEntryButton->setEnabled(!d->currentEntry.isNull() && d->tabWidget->currentIndex() == 0);
@@ -401,7 +413,7 @@ void SearchForm::startSearch()
 
         for (QMap<QListWidgetItem *, OnlineSearchAbstract *>::ConstIterator it = d->itemToOnlineSearch.constBegin(); it != d->itemToOnlineSearch.constEnd(); ++it)
             if (it.key()->checkState() == Qt::Checked) {
-                it.value()->startSearch();
+                it.value()->startSearchFromForm();
                 d->runningSearches.insert(it.value());
             }
         if (d->runningSearches.isEmpty()) {
@@ -418,25 +430,25 @@ void SearchForm::foundEntry(QSharedPointer<Entry> entry)
     d->sr->insertElement(entry);
 }
 
-void SearchForm::stoppedSearch(int resultCode)
+void SearchForm::stoppedSearch(int)
 {
     OnlineSearchAbstract *engine = static_cast<OnlineSearchAbstract *>(sender());
     if (d->runningSearches.remove(engine)) {
-        kDebug() << "Search from engine" << engine->label() << "stopped with code" << resultCode  << (resultCode == 0 ? "(OK)" : "(Error)");
         if (d->runningSearches.isEmpty()) {
             /// last search engine stopped
             d->switchToSearch();
             emit doneSearching();
 
-            QTimer::singleShot(1000, d->progressBar, SLOT(hide()));
-            QTimer::singleShot(1100, d->useEntryButton, SLOT(show()));
+            QTimer::singleShot(1000, d->progressBar, &QProgressBar::hide);
+            QTimer::singleShot(1100, d->useEntryButton, &QPushButton::show);
         } else {
             QStringList remainingEngines;
-            foreach (OnlineSearchAbstract *running, d->runningSearches) {
+            remainingEngines.reserve(d->runningSearches.size());
+            for (OnlineSearchAbstract *running : const_cast<const QSet<OnlineSearchAbstract *> &>(d->runningSearches)) {
                 remainingEngines.append(running->label());
             }
             if (!remainingEngines.isEmpty())
-                kDebug() << "Remaining running engines:" << remainingEngines.join(", ");
+                qCDebug(LOG_KBIBTEX_PROGRAM) << "Remaining running engines:" << remainingEngines.join(QStringLiteral(", "));
         }
     }
 }
@@ -456,9 +468,9 @@ void SearchForm::itemCheckChanged(QListWidgetItem *item)
 
     d->searchButton->setEnabled(numCheckedEngines > 0);
 
-    if (item != NULL) {
+    if (item != nullptr) {
         KConfigGroup configGroup(d->config, d->configGroupName);
-        QString name = item->data(NameRole).toString();
+        QString name = item->data(SearchForm::SearchFormPrivate::NameRole).toString();
         configGroup.writeEntry(name, item->checkState() == Qt::Checked);
         d->config->sync();
     }
